--- computations for simplex construction S

needsPackage "Polyhedra"

-- matrix of vertices determined using the construction given in the paper
S2 = matrix{{-1,1,-1},{-1,-1,3}}; -- 2d
S3 = matrix{{-1,1,-1,-1},{-1,-1,2,-1},{-1,-1,-1,11}}; -- 3d

-- determine matrix A with all-1-vector and nonnegative entries
matrixA = M -> (
    P = convexHull M;
    L = latticePoints P;
    L = matrix{L};
    n = numgens source L;
    d = numgens target L;
    seqn = 0..n-1;
    seqd = 0..d-1;
    listn = toList seqn;
    listd = toList seqd;
    listd = apply(listd, i -> min apply(listn, j -> L_(i,j)));
    T = matrix table(d,n,(i,j) -> -listd_i);
    L = L + T;
    E = transpose matrix vector(apply(listn, i -> 1));
    L = E||L
)

-- determine ml equations
MLeqs = {coeffRing=>QQ} >> o -> (expr) -> (
    m = numgens source expr;
    R := (o.coeffRing)[vars {53..(52+(numgens target expr))},u_1..u_m];
    A := substitute(expr,R);
    U = u_1..u_m;
    U = toList U;
    U = transpose matrix {for i in U list i_R};
    N = sum for i from 1 to m list u_i;
    X := for i in 0..(m-1) list R_(entries expr_i);
    X = transpose matrix {X};
    system := ideal (A*((N * X) - U));
    return first entries gens system
)

-- write list of polynomials in julia file  
SystemS2 = MLeqs matrixA S2
SystemS3 = MLeqs matrixA S3

polynomialjl = p -> (
    replace("_1","[1]",replace("_2","[2]",replace("_3","[3]",replace("_4","[4]",replace("_5","[5]",replace("_6","[6]",
    replace("_7","[7]",replace("_8","[8]",replace("_9","[9]",replace("_10","[10]",replace("_11","[11]",
    replace("_12","[12]",replace("_13","[13]",replace("_14","[14]",replace("_15","[15]",replace("_16","[16]",
    replace("_17","[17]",replace("_18","[18]",replace("_19","[19]",replace("_20","[20]",replace("_21","[21]",
    replace("_22","[22]",replace("_23","[23]",replace("_24","[24]",replace("_25","[25]",replace("_26","[26]",
    replace("_27","[27]",replace("_28","[28]",replace("_29","[29]",replace("_30","[30]",replace("_31","[31]",
    replace("_32","[32]",replace("_33","[33]",replace("_34","[34]",replace("_35","[35]",replace("_36","[36]",
    replace("_37","[37]",replace("_38","[38]",replace("_39","[39]",toExternalString p)))))))))))))))))))))))))))))))))))))))
)

writeSystem = method(Options=>{})

writeSystem (List, File) := o -> (P,f) -> (
    variables = {"x1","x2","x3","x4","u[1:39]"};
    f << "## This file was generated by Macaulay2 to compute the ML degree via homotopy continuation.\n\n";
    f << "using HomotopyContinuation";
    f << "\n\n";
    f << "@var " | concatenate(mingle(variables, #variables:1));
    eqnCommas:=(#P-1):", ";
    f << "\n\n";
    f << "F = System(\n";
    f << "    [\n";
    f << "         "|concatenate mingle(apply(P,e->polynomialjl e), eqnCommas);
    f << "\n";
    f << "    ];\n";
    f << "    parameters = u\n"  ;
    f << ")\n\n";
    f << "## Solve generically:\n\n";
    f << "u0 = rand(ComplexF64,39)\n";
    f << "result_u0 = solve(F, target_parameters = u0)\n\n";
    f << "## Solve for random parameter values\n\n";
    f << "data = [rand((0:100000),39)]\n";
    f << "data_points = solve(\n";
    f << "      F,\n";
    f << "      solutions(result_u0),\n";
    f << "      start_parameters =  u0,\n";
    f << "      target_parameters = data,\n";
    f << ")";
)

writeSystem (List, String) :=o -> (P,filename) -> (
    f := openOut filename;
    writeSystem(P,f);
    close f;
)

writeSystem(SystemS2, "simplexS2d.jl")
writeSystem(SystemS3, "simplexS3d.jl")